﻿using System;
using System.Windows.Input;
using System.Windows.Media;

namespace Petzold.Phone.Silverlight
{
    public class InertiaManager
    {
        // Private fields
        ManipulationInertiaDeltaEventArgs deltaEventArgs;
        double translationDeceleration;
        DateTime dtPrevious;
        
        // Public events
        public event EventHandler<ManipulationInertiaDeltaEventArgs> ManipulationInertiaDelta;
        public event EventHandler<ManipulationInertiaCompletedEventArgs> ManipulationInertiaCompleted;

        public InertiaManager(ManipulationCompletedEventArgs args) : base()
        {
            // Create one ManipulationInertiaDeltaEventArgs and re-use it
            deltaEventArgs = new ManipulationInertiaDeltaEventArgs
            {
                ManipulationContainer = args.ManipulationContainer,
                ManipulationOrigin = args.ManipulationOrigin,
                CumulativeManipulation = new ManipulationDelta(new Vector(args.TotalManipulation.Translation), 
                                                               new Vector(args.TotalManipulation.Scale))
            };

            // Initialize TranslationBehavior property
            this.TranslationBehavior = new InertiaTranslationBehavior();
            this.TranslationBehavior.InitialVelocity = new Vector(args.FinalVelocities.LinearVelocity);
        }

        public InertiaTranslationBehavior TranslationBehavior { get; set; }

        public bool Start()
        {
            if (!Double.IsNaN(TranslationBehavior.DesiredDisplacement))
            {
                double translationVelocityMagnitude = this.TranslationBehavior.InitialVelocity.Length;
                translationDeceleration = Math.Pow(translationVelocityMagnitude, 2) / (2 * TranslationBehavior.DesiredDisplacement);
            }
            else if (!Double.IsNaN(TranslationBehavior.DesiredDeceleration))
            {
                translationDeceleration = this.TranslationBehavior.DesiredDeceleration;
            }
            else
            {
                return false;
            }

            // Set the start velocity from the 
            deltaEventArgs.Velocities = new ManipulationVelocities(this.TranslationBehavior.InitialVelocity, new Vector());

            // Log the time and start the CompositionTarget.Rendering firings!
            dtPrevious = DateTime.Now;
            CompositionTarget.Rendering += OnCompositionTargetRendering;
            return true;
        }

        void OnCompositionTargetRendering(object sender, EventArgs args)
        {
            // Get elapsed time
            DateTime dtNow = DateTime.Now;
            TimeSpan timeSpan = dtNow - dtPrevious;
            dtPrevious = dtNow;

            // Break down the velocity into a magnitude and direction
            Vector translationVelocity = deltaEventArgs.Velocities.LinearVelocity;
            double translationVelocityMagnitude = translationVelocity.Length;
            Vector translationVelocityDirection = translationVelocity;
            translationVelocityDirection.Normalize();

            // Determine new velocity magnitude and velocity vector
            translationVelocityMagnitude = Math.Max(0, translationVelocityMagnitude - translationDeceleration * timeSpan.TotalSeconds);
            translationVelocity = translationVelocityMagnitude * translationVelocityDirection;

            // If the velocity magnitude is not zero (and Complete hasn't been called on the event args), fire another ManipulationIntertiaDelta event
            if (translationVelocityMagnitude != 0 && !deltaEventArgs.inertiaAborted)
            {
                Vector additionalTranslation = timeSpan.TotalSeconds * translationVelocity;
                deltaEventArgs.DeltaManipulation = new ManipulationDelta(additionalTranslation, new Vector());
                deltaEventArgs.CumulativeManipulation = new ManipulationDelta(deltaEventArgs.CumulativeManipulation.Translation + additionalTranslation, new Vector());
                deltaEventArgs.Velocities = new ManipulationVelocities(translationVelocity, new Vector());

                if (ManipulationInertiaDelta != null)
                    ManipulationInertiaDelta(this, deltaEventArgs);
            }
            // Otherwise fire a ManipulationInertiaCompleted event
            else
            {
                ManipulationInertiaCompletedEventArgs completedEventArgs = new ManipulationInertiaCompletedEventArgs();
                completedEventArgs.ManipulationContainer = deltaEventArgs.ManipulationContainer;
                completedEventArgs.ManipulationOrigin = deltaEventArgs.ManipulationOrigin;
                completedEventArgs.TotalManipulation = deltaEventArgs.CumulativeManipulation;
                completedEventArgs.FinalVelocities = new ManipulationVelocities(translationVelocity, new Vector());

                if (ManipulationInertiaCompleted != null)
                    ManipulationInertiaCompleted(this, completedEventArgs);

                // Turn off the CompositionTarget.Rendering event!
                CompositionTarget.Rendering -= OnCompositionTargetRendering;
            }
        }
    }
}
